	.include "pm_libs/pm_init.s"
	.include "pm_libs/pm_cstyle.s"

	pm_header "Blit Sprites", IRQ_KEY_POWER | IRQ_PRC_COPY, 0

	.include "pm_libs/pmrom_cstd.s"
	.include "pm_libs/pmrom_cspr.s"
	.include "pm_libs/pmrom_cmath.s"

	; Shutdown down when power key pressed
irq_key_power:
	cint CINT_SHUTDOWN

	; Flip map & sprites bases
	; This demo isn't over 64KB so we can skip upper address fliping
irq_prc_copy:
	pushax
	mov ba, [PRC_MAP_BASE]
	mov hl, [PRC_MAP_BASE_FLIP]
	mov [PRC_MAP_BASE], hl
	mov [PRC_MAP_BASE_FLIP], ba
	mov ba, [PRC_SPR_BASE]
	mov hl, [PRC_SPR_BASE_FLIP]
	mov [PRC_SPR_BASE], hl
	mov [PRC_SPR_BASE_FLIP], ba
	popax
	rirq IRQ_PRC_COPY

	; Globals
	.ram PRC_MAP_BASE_FLIP 2
	.ram PRC_SPR_BASE_FLIP 2
	.ram VRAM_BASE2 768

	; Main routine
main:
	; Setup PRC
	mov [n+PRC_MODE], PRC_ENABLE | PRC_MAP | PRC_MAP12x16 | PRC_SPR
	mov [n+PRC_RATE], PRC_36FPS
	mov ba, background_gfx1
	mov [PRC_MAP_BASE], ba
	mov ba, background_gfx2
	mov [PRC_MAP_BASE_FLIP], ba
	mov ba, sprites_gfx1
	mov [PRC_SPR_BASE], ba
	mov ba, sprites_gfx2
	mov [PRC_SPR_BASE_FLIP], ba

	; Unlock color registers, disable vram update
	colorpm_init
	mov [n+COLORPM_CMD], COLORPM_VRAM_OFF

	; This code can be used to detect if there's color interface:
	;   mov a, COLORPRC_UNLOCKED
	;   cmp a, [n+COLORPRC_CMD]
	;   jz colorprc_available

	; Enable interrupts
	enable_irqs IRQ_KEY_POWER | IRQ_PRC_COPY
	enable_mirq

	; Setup map and sprites
	c_memcpy MAP_BASE, background_map, 12*8

	; Draw the map with gray support
	cinvoke0 waitprc
	mov [n+PRC_MODE], PRC_ENABLE | PRC_MAP | PRC_MAP12x16
	cinvoke0 waitprc
	cinvoke0 flippage
	cinvoke0 waitprc
	cinvoke0 flippage
	and [n+PRC_MODE], ~PRC_MAP	; Clear map
	or [n+PRC_MODE], PRC_SPR	; Set sprites only

loop:
	; Clear sprites
	c_spr_clear

	; Add "Blit Sprites" overlay
	c_spr_add $1010, 0
	c_spr_add $1020, 1
	c_spr_add $1030, 2

	; Plot random sprite
	c_randf
	and a, 3
	add a, 4
	mov l, a	; Sprite 4 to 7
	mov h, 0
	c_randf
	and a, $7F	; random X pos, 0 ~ 127
	and b, $3F	; random Y pos, 0 ~ 64
	add ba, $1010	; Fix OAM position
	c_spr_add REG_BA, REG_HL

	; Blit sprites with gray support
	cinvoke0 waitprc
	cinvoke0 flippage
	c_spr_update
	cinvoke0 waitprc
	cinvoke0 flippage

	jmpb loop


	; Wait for PRC copy
	declare_function "waitprc", 0, 0
	begin_function
:		halt					; Battery likes HALT but be careful that it needs IRQs to wake up
		tst [n+IRQ_ACT1], IRQ_ACT1_PRC_DIV	; Wait for PRC divider flag, IRQ cannot be active for this trick to work
		jzb :b
		or [n+IRQ_ACT1], IRQ_ACT1_PRC_DIV	; Clear it
	end_function 0

	; Flip page
	declare_function "flippage", 0, 0
	begin_function
		mov x, VRAM_BASE
		mov y, VRAM_BASE2
		mov b, 0
:		mov l, [x]
		mov h, [y]
		mov [x], h
		mov [y], l
		inc x
		inc y
		mov l, [x]
		mov h, [y]
		mov [x], h
		mov [y], l
		inc x
		inc y
		mov l, [x]
		mov h, [y]
		mov [x], h
		mov [y], l
		inc x
		inc y
		jdbnz :b
		mov [n+COLORPM_CMD], COLORPM_FLIP
	end_function 0

	pm_rominfo	; End of code

background_map:
	.incbin "background.map"

	.org 16384	; It's best to have a fixed place for color graphics
			; or we will easily end up with corrupted color info.

	pm_align_map
background_gfx1:
	.incbin "background_gfx1.bin"
background_gfx2:
	.incbin "background_gfx2.bin"

	pm_align_spr
sprites_gfx1:
	.incbin "sprites_gfx1.bin"
sprites_gfx2:
	.incbin "sprites_gfx2.bin"
