	.include "pm_libs/pm_init.s"
	.include "pm_libs/pm_cstyle.s"

	pm_header "TestFramebuf", IRQ_KEY_POWER | IRQ_PRC_COPY | IRQ_MUSIC | IRQ_SHOCK, 0

	.include "pm_libs/pmrom_cstd.s"
	.include "pm_libs/pmrom_cmath.s"
	.include "pm_libs/pmrom_cmusic.s"

	; For grayscaling (3rd color)
	.ram PRC_MAP_BASE_FLIP 2
	.ram PRC_SPR_BASE_FLIP 2

	; All coords are fixed-point 8.8
	.ram pokeball_px 2	; Position
	.ram pokeball_py 2
	.ram pokeball_vx 2	; Velocity
	.ram pokeball_vy 2
	.ram pokeball_ax 2	; Acceleration
	.ram pokeball_ay 2
	.ram pokeball_anim 2	; Rolling animation

	.set pokeball_reducex	8
	.set pokeball_reducey	64

	; Power down when power key pressed
irq_key_power:
	cint CINT_SHUTDOWN

	; Flip map & spr bases
	; This demo isn't over 64KB so we can skip upper address fliping
irq_prc_copy:
	pushax
	mov ba, [PRC_SPR_BASE]
	mov hl, [PRC_SPR_BASE_FLIP]
	mov [PRC_SPR_BASE], hl
	mov [PRC_SPR_BASE_FLIP], ba
	mov ba, [PRC_MAP_BASE]
	mov hl, [PRC_MAP_BASE_FLIP]
	mov [PRC_MAP_BASE], hl
	mov [PRC_MAP_BASE_FLIP], ba
	popax
	rirq IRQ_PRC_COPY

	; Shock detector handler
irq_shock:
	pushax
	mov hl, [pokeball_py]
	cmp hl, 46<<8		; If near ground...
	jl :f
	mov hl, [pokeball_vy]
	cmp hl, $8000		; and if going up...
	jnc :f
	cinvoke0 onshock
:	popax
	rirq IRQ_SHOCK

	declare_function "onshock", 0, 0
	begin_function
		; Project ball way up
		add hl, 2<<8
		mov [pokeball_vy], hl
		; Throw it left or right based on luck
		c_randf
		mov hl, [pokeball_vx]
		tst a, 0x01
		jnzb :f
		add hl, 1<<8
		jmpb :ff
:		sub hl, 1<<8
:		mov [pokeball_vx], hl
	end_function 0

	; Main routine
main:
	; Setup ball
	cinvoke0 initball

	; Copy background map
	c_memcpy MAP_BASE, background_map, 96

	; Setup map and sprite bases
	; This demo isn't over 64KB so we don't care about the upper address
	mov ba, background_1
	mov [PRC_MAP_BASE], ba
	mov ba, background_2
	mov [PRC_MAP_BASE_FLIP], ba
	mov ba, pokeball_1
	mov [PRC_SPR_BASE], ba
	mov ba, pokeball_2
	mov [PRC_SPR_BASE_FLIP], ba

	; Setup PRC
	mov [n+PRC_MODE], PRC_ENABLE | PRC_SPR | PRC_MAP | PRC_MAP12x16
	mov [n+PRC_RATE], PRC_36FPS

	; Enable interrupts
	enable_irqs IRQ_KEY_POWER | IRQ_PRC_COPY | IRQ_SHOCK
	enable_mirq

	; Initialize pmmusic for our sound effect
	c_music_init $00FF, 0

	; Begin loop
:	cinvoke0 wait_prc
	cinvoke0 update_position
	cinvoke0 update_velocity
	cinvoke0 update_anim
	cinvoke0 update_sprite
	jmpb :b

	; Wait PRC div
	declare_function "wait_prc", 0, 0
	begin_function
:		halt					; Battery likes HALT but be careful that it needs IRQs to wake up
		tst [n+IRQ_ACT1], IRQ_ACT1_PRC_DIV	; Wait for PRC divider flag, IRQ cannot be active for this trick to work
		jzb :b
		or [n+IRQ_ACT1], IRQ_ACT1_PRC_DIV	; Clear it
	end_function 0

	; Initialize ball
	declare_function "initball", 0, 0
	begin_function
		mov ba, 40<<8
		mov [pokeball_px], ba
		mov ba, 24<<8
		mov [pokeball_py], ba
		mov ba, 64
		mov [pokeball_vx], ba
		mov ba, -64
		mov [pokeball_vy], ba
		mov ba, 0
		mov [pokeball_ax], ba
		mov ba, 16
		mov [pokeball_ay], ba
		mov a, 0
		mov [pokeball_anim], ba
	end_function 0

	; Copy pokeball information into sprite 0
	declare_function "update_sprite", 0, 0
	begin_function
		mov a, [pokeball_py+1]
		add a, 16				; HW Sprites start at X of  -16
		mov [SPR_BASE + SPR_Y], a
		mov a, [pokeball_px+1]
		add a, 16				; HW Sprites start at Y of -16
		mov [SPR_BASE + SPR_X], a
		mov a, [pokeball_anim+1]		; Use the integer part of our animation
		mov [SPR_BASE + SPR_TILE], a
		mov a, SPR_ENABLE
		mov [SPR_BASE + SPR_CTRL], a
	end_function 0

	; Update ball position based of velocity
	declare_function "update_position", 0, 0
	begin_function
		; Read velocity
		mov x, [pokeball_vx]
		mov y, [pokeball_vy]

		; X coord.
		mov ba, [pokeball_px]
		add ba, x		; px += vx;
		cmp ba, 0<<8		; if (px < 0.0) {
		jge :f
		mov ba, 0<<8		;	px = 0.0;
		cmp x, $8000
		js :f
		mov hl, 0
		sub hl, x
		mov x, hl		;	vx = abs(vx);
		push ba			;	oncollision(); }
		cinvoke2 oncollision, REG_X, REG_Y_DESTROY
		mov y, ba
		pop ba

:		cmp ba, 80<<8		; if (px >= 80.0) {
		jle :f
		mov ba, 80<<8		;	px = 80.0;
		cmp x, $8000
		jns :f
		mov hl, 0
		sub hl, x
		mov x, hl		;	vx = -abs(vx);
		push ba			;	oncollision(); }
		cinvoke2 oncollision, REG_X, REG_Y_DESTROY
		mov y, ba
		pop ba
:		mov [pokeball_px], ba

		; Y coord.
		mov ba, [pokeball_py]
		add ba, y		; py += vy;
		cmp ba, 0<<8		; if (py < 0.0) {
		jge :f
		mov ba, 0<<8		;	py = 0.0;
		cmp y, $8000
		js :f
		mov hl, 0
		sub hl, y
		mov y, hl		;	vy = abs(vy);
		push ba			;	oncollision(); }
		cinvoke2 oncollision, REG_X, REG_Y_DESTROY
		mov y, ba
		pop ba

:		cmp ba, 48<<8		; if (py >= 48.0) {
		jle :f
		mov ba, 48<<8		;	py = 48.0;
		mov y, [pokeball_vy]
		cmp y, $8000
		jns :f
		mov hl, 0
		sub hl, y
		mov y, hl		;	vy = -abs(vy);
		push ba			;	oncollision(); }
		cinvoke2 oncollision, REG_X, REG_Y_DESTROY
		mov y, ba
		pop ba
:		mov [pokeball_py], ba

		; Update velocity
		mov [pokeball_vx], x
		mov [pokeball_vy], y
	end_function 0

	; Update ball velocity based of acceleration
	declare_function "update_velocity", 0, 0
	begin_function
		; X velocity
		mov hl, [pokeball_ax]
		mov ba, [pokeball_vx]
		add ba, hl		; vx += ax;
		cmp ba, -3<<8		; if (vx < -3.0) vx = -3.0;
		jge :f
		mov ba, -3<<8
:		cmp ba, 3<<8		; if (vx > 3.0) vx = 3.0;
		jle :f
		mov ba, 3<<8
:		mov [pokeball_vx], ba

		; Y velocity
		mov hl, [pokeball_ay]
		mov ba, [pokeball_vy]
		add ba, hl		; vy += ay;
		cmp ba, -3<<8		; if (vy < -3.0) vx = -3.0;
		jge :f
		mov ba, -3<<8
:		cmp ba, 3<<8		; if (vy > 3.0) vx = 3.0;
		jle :f
		mov ba, 3<<8
:		mov [pokeball_vy], ba
	end_function 0

	; Update animation based of velocity
	declare_function "update_anim", 0, 0
	begin_function
		mov ba, [pokeball_anim]
		mov hl, [pokeball_vx]
		add ba, hl
		and b, $07		; anim = (anim + vx) & 0x07FF;
		mov [pokeball_anim], ba
	end_function 0

	; Called when the ball collide with any wall
	; Return:
	;  BA = Deduced Y velocity
	;   X = Deduced X velocity
	declare_function "oncollision", 0, 2
	begin_function
		getparam 0, REG_X
		getparam 1, REG_BA

		; Deduce X velocity
		cmp x, $8000
		jncb _xneg
_xpos:
		sub x, pokeball_reducex
		jnc :f
		mov x, 0
		jmpb :f
_xneg:
		add x, pokeball_reducex
		jnc :f
		mov x, 0
:		mov [pokeball_vx], x

		; Deduce X velocity
		cmp ba, $8000
		jncb _yneg
_ypos:
		sub ba, pokeball_reducey
		jnc :f
		mov ba, 0
		jmpb :f
_yneg:
		add ba, pokeball_reducey
		jnc :f
		mov ba, 0
:		mov [pokeball_vy], ba

		; Only play SFX if ball is moving
		cmp x, 0
		jnzb _do_sfx
		cmp ba, 0
		jnzb _do_sfx
		return
_do_sfx:
		push x
		push ba
		c_music_playsfx wallsfx
		pop ba
		pop x
	end_function 0


	; PMMusic stuff here!
	pmmusic_align
wallsfx:
	;           Note    Pw  V  Wait
	pmmusic_row N_C_2, $20, 2, 1
	pmmusic_row N_C_7, $80, 2, 1
	pmmusic_row N_C_5, $40, 2, 1
	pmmusic_row N_C_3, $20, 2, 1
	pmmusic_end


	; Tiles must be 8 bytes aligned!
	pm_align_map
background_1:
	.incbin "background.bin"
background_2:
	.incbin "backgroundG.bin"
background_map:
	.incbin "background.map"


	; Sprites must be 64 bytes aligned!
	pm_align_spr
pokeball_1:
	.incbin "pokeball.bin"
pokeball_2:
	.incbin "pokeballG.bin"


	pm_rominfo
