	.include "pm_libs/pm_init.s"
	.include "pm_libs/pm_cstyle.s"

	pm_header "TestMapView ", IRQ_KEY_POWER | IRQ_PRC_COPY, 0

	.include "pm_libs/pmrom_cstd.s"
	.include "pm_libs/pmrom_cmap16.s"

	; Shutdown down when power key pressed
irq_key_power:
	cint CINT_SHUTDOWN

	; Flip map bases
	; This demo isn't over 64KB so we can skip upper address fliping
irq_prc_copy:
	pushax
	mov ba, [PRC_MAP_BASE]
	mov hl, [PRC_MAP_BASE_FLIP]
	mov [PRC_MAP_BASE], hl
	mov [PRC_MAP_BASE_FLIP], ba
	popax
	rirq IRQ_PRC_COPY

	; Globals
	.ram PRC_MAP_BASE_FLIP 2

	; Main routine
main:
	; Setup PRC
	mov [n+PRC_MODE], PRC_ENABLE | PRC_MAP | PRC_MAP16x12
	mov [n+PRC_RATE], PRC_36FPS
	mov ba, tileset_1
	mov [PRC_MAP_BASE], ba
	mov ba, tileset_2
	mov [PRC_MAP_BASE_FLIP], ba

	; Enable interrupts
	enable_irqs IRQ_KEY_POWER | IRQ_PRC_COPY
	enable_mirq

	; Set map
	c_map16_set pokemap16_mapw, pokemap16_maph, pokemap16

loop:
	; Battery likes HALT but be careful that it needs IRQs to wake up
	halt

	; Render map
	c_map16_draw

	; Move
	cinvoke0 movemap

	jmpb loop


	; Move around the map
	declare_function "movemap", 0, 0
	varlocal _camera_speed, 2
	begin_function
		; Default speed
		setlocalw _camera_speed, 8

		c_keypad

		; Speed modifiers
		tst a, KEY_B
		callnzb _keyb
		tst a, KEY_A
		callnzb _keya
		tst a, KEY_C
		callnzb _keyc

		; Move the camera
		tst a, KEY_LEFT
		callnzw _left
		tst a, KEY_RIGHT
		callnzw _right
		tst a, KEY_UP
		callnzw _up
		tst a, KEY_DOWN
		callnzw _down
		return

_keyb:
		setlocalw _camera_speed, 32
		ret
_keya:
		setlocalw _camera_speed, 4
		ret
_keyc:
		setlocalw _camera_speed, 64
		ret
_left:
		push ba
		getlocalw _camera_speed, REG_BA
		mov hl, 0
		sub hl, ba
		c_map16_movex REG_HL
		pop ba
		ret
_right:
		push ba
		c_map16_movex LOCAL_OF + _camera_speed
		pop ba
		ret
_up:
		push ba
		getlocalw _camera_speed, REG_BA
		mov hl, 0
		sub hl, ba
		c_map16_movey REG_HL
		pop ba
		ret
_down:
		push ba
		c_map16_movey LOCAL_OF + _camera_speed
		pop ba
		ret
	end_function 0

pokemap16:
	.include "pokemap16.inc"
	.incbin "pokemap16.map"

	pm_align_map
tileset_1:
	.incbin "tileset16.bin"
tileset_2:
	.incbin "tileset16G.bin"

	pm_rominfo
