	.include "pm_libs/pm_init.s"
	.include "pm_libs/pm_cstyle.s"

	pm_header "PMMusic_TsTC", IRQ_KEY_POWER | IRQ_MUSIC | IRQ_PRC_COPY, 0

	.include "pm_libs/pmrom_cstd.s"
	.include "pm_libs/pmrom_cmusic.s"

	.ram	prc_tick	1
	.ram	menu_cursor	1
	.ram	bgm_select	1
	.ram	sfx_select	1

	; Power button, shutdown PM
irq_key_power:
	cint CINT_SHUTDOWN

	; Act as VSync counter
irq_prc_copy:
	push hl
	mov hl, prc_tick
	inc [hl]
	pop hl
	rirq IRQ_PRC_COPY

main_screen:
	.db "Sound TsT -f"
	.db "  BGM 00    "
	.db "  SFX 00    "
	.db "  Vol BGM ? "
	.db "  Vol SFX ? "
	.db "A= B= C=FX"
	.db ""
	.db "            "

	; Main routine
main:
	; Initialize RAM
	c_memset RAM_BASE, 0, 4

	; Interrupts
	enable_irqs IRQ_KEY_POWER | IRQ_PRC_COPY

	; PRC
	mov [n+PRC_MODE], PRC_ENABLE | PRC_MAP | PRC_MAP12x16
	mov [n+PRC_RATE], PRC_RATE_2
	mov hl, font8x8
	mov [PRC_MAP_BASE], hl

	; Initialize menu
	c_memcpy MAP_BASE, main_screen, 96
	cinvoke0 render_menu_updatebgmname

	; Initialize sound engine
	c_music_init $00FF, MAP_BASE

	; Enable interrupt
	enable_mirq

:	halt
	c_keypad
	cmp a, 0
	jnz :b
:	cinvoke0 render_menu
	cinvoke0 proc_menu
	cinvoke0 wait_prc
	jmp :b

	; Wait PRC
	declare_function "wait_prc", 0, 0
	begin_function
		mov a, [prc_tick]
:		halt
		mov b, [prc_tick]
		cmp a, b
		jz :b
	end_function 0

	; Wait PRC (Num)
	declare_function "wait_prc_num", 0, 1
	begin_function
		getparam 0, REG_L
:		cinvoke0 wait_prc
		dec l
		jnz :b
	end_function 0

	; Render menu
	declare_function "render_menu", 0, 0
	begin_function
		; Get BGM selection and write to map
		mov a, [bgm_select]
		unpack
		mov b, 0
		mov hl, hexnum
		add hl, ba
		mov a, [hl]
		mov [MAP_BASE + (1 * 12) + 7], a
		mov a, [bgm_select]
		unpack
		mov a, b
		mov b, 0
		mov hl, hexnum
		add hl, ba
		mov a, [hl]
		mov [MAP_BASE + (1 * 12) + 6], a
		; Get SFX selection and write to map
		mov a, [sfx_select]
		unpack
		mov b, 0
		mov hl, hexnum
		add hl, ba
		mov a, [hl]
		mov [MAP_BASE + (2 * 12) + 7], a
		mov a, [sfx_select]
		unpack
		mov a, b
		mov b, 0
		mov hl, hexnum
		add hl, ba
		mov a, [hl]
		mov [MAP_BASE + (2 * 12) + 6], a
		; Get BGM volume and write to map
		c_music_getvolbgm
		mov hl, hexnum
		add hl, ba
		mov a, [hl]
		mov [MAP_BASE + (3 * 12) + 10], a
		; Get SFX volume and write to map
		c_music_getvolsfx
		mov hl, hexnum
		add hl, ba
		mov a, [hl]	
		mov [MAP_BASE + (4 * 12) + 10], a
		; Check for playing sounds
		mov l, ' '
		c_music_isplayingbgm
		cmp ba, 0
		jz :f
		mov l, ''
:		mov [MAP_BASE + (1 * 12) + 9], l
		mov l, ' '
		c_music_isplayingsfx
		cmp ba, 0
		jz :f
		mov l, ''
:		mov [MAP_BASE + (2 * 12) + 9], l
	end_function 0

	; Process menu
	declare_function "proc_menu", 0, 0
	begin_function
		c_keypad
		tst a, KEY_UP
		jnzw _up
		tst a, KEY_DOWN
		jnzw _down
		tst a, KEY_A
		jnzw _play
		tst a, KEY_B
		jnzw _stop
		tst a, KEY_C
		jnzw _sfx
		jmpw _menuexec
_up:
		mov hl, menu_cursor
		cmp [hl], 0
		jzw _menuexec
		dec [hl]
		cinvoke1 wait_prc_num, 8
		jmpw _menuexec
_down:
		mov hl, menu_cursor
		cmp [hl], 3
		jzw _menuexec
		inc [hl]
		cinvoke1 wait_prc_num, 8
		jmpw _menuexec
_play:
		mov b, 0
		mov a, [bgm_select]
		shl a
		shl a
		shl a
		shl a
		mov hl, bgm_list+2
		add hl, ba
		mov a, [hl]
		mov xi, a
		dec hl
		dec hl
		mov x, [hl]
		c_music_playbgm, PTR_FX
		cinvoke0 render_menu
		cinvoke1 wait_prc_num, 8
		jmpw _menuexec
_stop:
		c_music_stopbgm
		cinvoke0 render_menu
		cinvoke1 wait_prc_num, 8
		jmpw _menuexec
_sfx:
		mov b, 0
		mov a, [sfx_select]
		shl a
		shl a
		mov hl, sfx_list+2
		add hl, ba
		mov a, [hl]
		mov xi, a
		dec hl
		dec hl
		mov x, [hl]
		c_music_playsfx, PTR_FX
		cinvoke0 render_menu
		cinvoke1 wait_prc_num, 8
_menuexec:
		mov a, [menu_cursor]
		cmp a, 0
		jnzb :f
			cinvoke0 proc_menu_0
			return
:		cmp a, 1
		jnzb :f
			cinvoke0 proc_menu_1
			return
:		cmp a, 2
		jnzb :f
			cinvoke0 proc_menu_2
			return
:		cmp a, 3
		jnzb :f
			cinvoke0 proc_menu_3
:		
	end_function 0

	; BGM select options
	declare_function "proc_menu_0", 0, 0
	begin_function
		; Set cursor
		xor a, a
		mov b, $1A
		mov [MAP_BASE + (1 * 12) + 1], b
		mov [MAP_BASE + (2 * 12) + 1], a
		mov [MAP_BASE + (3 * 12) + 1], a
		mov [MAP_BASE + (4 * 12) + 1], a
		; Process keys
		c_keypad
		tst a, KEY_LEFT
		jnz _prevbgm
		tst a, KEY_RIGHT
		jnz _nextbgm
		return
_prevbgm:
		mov hl, bgm_select
		cmp [hl], 0
		returnc z
		dec [hl]
		cinvoke0 render_menu_updatebgmname
		cinvoke1 wait_prc_num, 8
		return
_nextbgm:
		mov hl, bgm_select
		mov a, [bgm_list_last]
		cmp [hl], a
		returnc z
		inc [hl]
		cinvoke0 render_menu_updatebgmname
		cinvoke1 wait_prc_num, 8
	end_function 0

	; SFX select options
	declare_function "proc_menu_1", 0, 0
	begin_function
		; Set cursor
		xor a, a
		mov b, $1A
		mov [MAP_BASE + (1 * 12) + 1], a
		mov [MAP_BASE + (2 * 12) + 1], b
		mov [MAP_BASE + (3 * 12) + 1], a
		mov [MAP_BASE + (4 * 12) + 1], a
		; Process keys
		c_keypad
		tst a, KEY_LEFT
		jnz _prevsfx
		tst a, KEY_RIGHT
		jnz _nextsfx
		return
_prevsfx:
		mov hl, sfx_select
		cmp [hl], 0
		returnc z
		dec [hl]
		cinvoke1 wait_prc_num, 8
		return
_nextsfx:
		mov hl, sfx_select
		mov a, [sfx_list_last]
		cmp [hl], a
		returnc z
		inc [hl]
		cinvoke1 wait_prc_num, 8
	end_function 0

	; BGM volume options
	declare_function "proc_menu_2", 0, 0
	begin_function
		; Set cursor
		xor a, a
		mov b, $1A
		mov [MAP_BASE + (1 * 12) + 1], a
		mov [MAP_BASE + (2 * 12) + 1], a
		mov [MAP_BASE + (3 * 12) + 1], b
		mov [MAP_BASE + (4 * 12) + 1], a
		; Process keys
		c_keypad
		tst a, KEY_LEFT
		jnz _decvol
		tst a, KEY_RIGHT
		jnz _incvol
		return
_decvol:
		c_music_getvolbgm
		dec a
		c_music_setvolbgm REG_BA
		cinvoke1 wait_prc_num, 8
		return
_incvol:
		c_music_getvolbgm
		inc a
		c_music_setvolbgm REG_BA
		cinvoke1 wait_prc_num, 8
	end_function 0

	; SFX volume options
	declare_function "proc_menu_3", 0, 0
	begin_function
		; Set cursor
		xor a, a
		mov b, $1A
		mov [MAP_BASE + (1 * 12) + 1], a
		mov [MAP_BASE + (2 * 12) + 1], a
		mov [MAP_BASE + (3 * 12) + 1], a
		mov [MAP_BASE + (4 * 12) + 1], b
		; Process keys
		c_keypad
		tst a, KEY_LEFT
		jnz _decvol
		tst a, KEY_RIGHT
		jnz _incvol
		return
_decvol:
		c_music_getvolsfx
		dec a
		c_music_setvolsfx REG_BA
		cinvoke1 wait_prc_num, 8
		return
_incvol:
		c_music_getvolsfx
		inc a
		c_music_setvolsfx REG_BA
		cinvoke1 wait_prc_num, 8
	end_function 0

	; Transfer BGM name to screen
	declare_function "render_menu_updatebgmname", 0, 0
	begin_function
		mov a, [bgm_select]
		unpack
		swap a
		mov hl, bgm_list
		add hl, ba
		add hl, 4
		c_memcpy MAP_BASE + (7 * 12), PTR_ZHL, 12
	end_function 0

; --------------------------------------------------------------------------------

hexnum:
	.db "0123456789ABCDEF"

	.align 16
bgm_list:
	.dd music_test
	.db "SndEng Tst  "
	.dd music_pmintro
	.db "PM Intro    "
	.dd music_choco
	.db "OdekaChocobo"
	.dd bgm_unk_tune1
	.db "Unk. Tune 1 "
	.dd bgm_unk_tune2
	.db "Unk. Tune 2 "
	.dd music_galactix
	.db "Galactix  1P"
	.dd music_galactix2
	.db "GalactixRm1P"
	.dd music_metroid
	.db "MetroidTitle"
sfx_list:
	.dd sound_fx0
	.dd sound_fx1
	.dd sound_fx2
	.dd sound_fx3
	.dd sound_fx4
	.dd sound_fx5

bgm_list_last:	.db 7
sfx_list_last:	.db 5

	PMMUSIC_ALIGN

music_data_start:

.include "music_test.asm"
.include "music_pmintro.asm"
.include "music_choco.asm"
.include "music_unknown.asm"
.include "music_galactix.asm"
.include "music_metroid.asm"

sound_fx0:
	PMMUSIC_ROW N_C_4, $80, 3, 1
	PMMUSIC_ROW N_C_3, $80, 3, 1
	PMMUSIC_ROW N_C_5, $80, 3, 1
	PMMUSIC_ROW N_C_7, $80, 3, 1
	PMMUSIC_END

sound_fx1:
	PMMUSIC_ROW N_C_4, $80, 3, 1
	PMMUSIC_ROW N_C_5, $70, 3, 1
	PMMUSIC_ROW N_C_4, $60, 3, 1
	PMMUSIC_ROW N_C_5, $50, 3, 1
	PMMUSIC_ROW N_C_4, $40, 3, 1
	PMMUSIC_ROW N_C_5, $30, 3, 1
	PMMUSIC_ROW N_C_4, $20, 3, 1
	PMMUSIC_ROW N_C_5, $10, 3, 1
	PMMUSIC_ROW N_C_4, $00, 3, 1
	PMMUSIC_END

sound_fx2:
	PMMUSIC_ROW N_C_1, $80, 3, 1
	PMMUSIC_ROW N_E_1, $80, 3, 1
	PMMUSIC_ROW N_G_1, $80, 3, 1
	PMMUSIC_ROW N_C_2, $80, 3, 1
	PMMUSIC_ROW N_E_2, $80, 3, 1
	PMMUSIC_ROW N_G_2, $80, 3, 1
	PMMUSIC_ROW N_C_3, $80, 3, 1
	PMMUSIC_ROW N_E_3, $80, 3, 1
	PMMUSIC_ROW N_G_3, $80, 3, 1
	PMMUSIC_ROW N_C_4, $80, 3, 1
	PMMUSIC_ROW N_E_4, $80, 3, 1
	PMMUSIC_ROW N_G_4, $80, 3, 1
	PMMUSIC_END

sound_fx3:
	PMMUSIC_ROW N_C_4, $40, 0, 1
	PMMUSIC_ROW N_C_4, $40, 3, 1
	PMMUSIC_ROW N_C_3, $40, 2, 1
	PMMUSIC_ROW N_C_2, $40, 2, 1
	PMMUSIC_ROW N_C_1, $40, 2, 1
	PMMUSIC_ROW N_C_1, $40, 2, 1
	PMMUSIC_END

sound_fx4:
	PMMUSIC_ROW N_A_5, $40, 2, 6
	PMMUSIC_WRITERAM (0 * 12 + 10), $2B
	PMMUSIC_ROW N_A_5, $20, 2, 6
	PMMUSIC_WRITERAM (0 * 12 + 10), $2D
	PMMUSIC_END

sound_fx5:
	PMMUSIC_ROW N_E_6, $80, 3, 1
	PMMUSIC_ROW N_E_4, $70, 2, 1
	PMMUSIC_ROW N_E_5, $60, 3, 1
	PMMUSIC_ROW N_E_3, $50, 2, 1
	PMMUSIC_ROW N_E_4, $40, 3, 1
	PMMUSIC_ROW N_E_2, $30, 2, 1
	PMMUSIC_ROW N_E_1, $20, 2, 1
	PMMUSIC_END

music_data_end:
	.printf "Music Data size: %i bytes\n", music_data_end - music_data_start

; --------------------------------------------------------------------------------

	pm_align_tiles
font8x8:
	.incbin "pm_libs/font8x8.bin"

	pm_rominfo
