	.include "pm_libs/pminit.s"
	.include "pm_libs/pmmusic_macro.s"

	RAMBEGIN
	.include "pm_libs/pmmusic_ram.s"
prc_tick:	.ds 1
menu_cursor:	.ds 1
bgm_select:	.ds 1
sfx_select:	.ds 1
	RAMEND

	HEADER "PMMT", "PMMusic_TsTC", IRQ_KEY_POWER | IRQ_TMR2_HI | IRQ_PRC_COPY, 0

	; Power button, shutdown PM
irq_key_power:
	cint CINT_SHUTDOWN

	; Act as VSync counter
irq_prc_copy:
	push hl
	mov hl, prc_tick
	inc [hl]
	pop hl
	rirq RIRQ_PRC_COPY

	.include "pm_libs/pmmusic_rom.s"

main_screen:
	.db "Sound TsT -e"
	.db "  BGM 00    "
	.db "  SFX 00    "
	.db "  Vol BGM 3 "
	.db "  Vol SFX 3 "
	.db "A= B= C=FX"
	.db ""
	.db "            "

	; Main routine
main:
	pm_init

	; Initialize RAM
	xor a, a
	mov [prc_tick], a
	mov [menu_cursor], a
	mov [bgm_select], a
	mov [sfx_select], a

	; Interrupts
	mov [n+IRQ_ENA3], IRQ_ENA3_KEY_POWER
	mov [n+IRQ_PRI2], IRQ_PRI2_KEY_PAD
	mov [n+IRQ_ENA1], IRQ_ENA1_PRC_COPY
	mov [n+IRQ_PRI1], IRQ_PRI1_PRC

	; PRC
	mov [n+PRC_MODE], PRC_ENABLE | PRC_MAP | PRC_MAP12x16
	mov [n+PRC_RATE], PRC_RATE_2
	mov hl, font8x8
	mov [PRC_MAP_BASE], hl
	mov x, main_screen
	mov y, MAP_BASE
	mov b, 96
mainscreencopy:
	mov a, [x]
	mov [y], a
	inc x
	inc y
	jdbnz mainscreencopy
	call render_menu_updatebgmname

	; Init sound engine and play music (BA = Master time, HL = RAM Ptr)
	mov ba, MAP_BASE
	mov hl, $00FF
	call pmmusic_init

	; Enable interrupt
	mov f, ENABLE_IRQ

displayloop:
	call wait_prc
	call read_keys
	cmp a, 0
	jnz displayloop
displayloop2:
	call render_menu
	call proc_menu
	call wait_prc
	jmp displayloop2

	; Read Keypad
	; Return A, 0 = Released, 1 = Pressed
read_keys:
	mov a, [n+KEY_PAD]
	push b
	mov b, [n+KEY_PAD]
	xor a, b
	mov a, b
	pop b
	jnz read_keys
	not a
	ret

	; Wait PRC
wait_prc:
	push ba
	mov a, [prc_tick]
wait_prc_loop:
	halt
	mov b, [prc_tick]
	cmp a, b
	jz wait_prc_loop
	pop ba
	ret

	; Wait PRC (Num)
	; A = Number of copies
wait_prc_num:
	push ba
	mov b, a
wait_prc_num_loop:
	call wait_prc
	jdbnz wait_prc_num_loop
	pop ba
	ret

render_menu:
	; Get BGM selection and write to map
	mov a, [bgm_select]
	unpack
	mov b, 0
	mov hl, hexnum
	add hl, ba
	mov a, [hl]
	mov [MAP_BASE + (1 * 12) + 7], a
	mov a, [bgm_select]
	unpack
	mov a, b
	mov b, 0
	mov hl, hexnum
	add hl, ba
	mov a, [hl]
	mov [MAP_BASE + (1 * 12) + 6], a
	; Get SFX selection and write to map
	mov a, [sfx_select]
	unpack
	mov b, 0
	mov hl, hexnum
	add hl, ba
	mov a, [hl]
	mov [MAP_BASE + (2 * 12) + 7], a
	mov a, [sfx_select]
	unpack
	mov a, b
	mov b, 0
	mov hl, hexnum
	add hl, ba
	mov a, [hl]
	mov [MAP_BASE + (2 * 12) + 6], a
	ret

proc_menu:
	call read_keys
	tst a, KEY_UP
	jnz proc_menu_up
	tst a, KEY_DOWN
	jnz proc_menu_down
	tst a, KEY_A
	jnz proc_menu_play
	tst a, KEY_B
	jnz proc_menu_stop
	tst a, KEY_C
	jnz proc_menu_sfx
proc_menu2:
	mov a, [menu_cursor]
	cmp a, 0
	jz proc_menu_0
	cmp a, 1
	jz proc_menu_1
	cmp a, 2
	jz proc_menu_2
	cmp a, 3
	jz proc_menu_3
	ret

proc_menu_up:
	mov hl, menu_cursor
	cmp [hl], 0
	jz proc_menu2
	dec [hl]
	mov a, 8
	call wait_prc_num
	jmp proc_menu2

proc_menu_down:
	mov hl, menu_cursor
	cmp [hl], 3
	jz proc_menu2
	inc [hl]
	mov a, 8
	call wait_prc_num
	jmp proc_menu2

proc_menu_play:
	mov b, 0
	mov a, [bgm_select]
	shl a
	shl a
	shl a
	shl a
	mov hl, bgm_list+2
	add hl, ba
	mov b, [hl]
	dec hl
	dec hl
	mov hl, [hl]
	call pmmusic_playbgm
	mov a, 8
	call wait_prc_num
	jmp proc_menu2

proc_menu_stop:
	call pmmusic_stopbgm
	mov a, 8
	call wait_prc_num
	jmp proc_menu2

proc_menu_sfx:
	mov b, 0
	mov a, [sfx_select]
	shl a
	shl a
	mov hl, sfx_list+2
	add hl, ba
	mov b, [hl]
	dec hl
	dec hl
	mov hl, [hl]
	call pmmusic_playsfx
	mov a, 8
	call wait_prc_num
	jmp proc_menu2

do_ret:
	ret

proc_menu_0:
	; Set cursor
	xor a, a
	mov b, $1A
	mov [MAP_BASE + (1 * 12) + 1], b
	mov [MAP_BASE + (2 * 12) + 1], a
	mov [MAP_BASE + (3 * 12) + 1], a
	mov [MAP_BASE + (4 * 12) + 1], a
	; Process keys
	call read_keys
	tst a, KEY_LEFT
	jnz proc_menu_0_prevbgm
	tst a, KEY_RIGHT
	jnz proc_menu_0_nextbgm
	ret

render_menu_updatebgmname:
	; Transfer BGM name to screen
	mov a, [bgm_select]
	unpack
	swap a
	mov hl, bgm_list
	add hl, ba
	add hl, 4
	mov x, MAP_BASE + (7 * 12)
	mov b, 12
render_menu_cloop:
	mov a, [hl]
	mov [x], a
	inc hl
	inc x
	jdbnz render_menu_cloop
	ret

proc_menu_0_prevbgm:
	mov hl, bgm_select
	cmp [hl], 0
	jz do_ret
	dec [hl]
	mov a, 8
	call wait_prc_num
	call render_menu_updatebgmname
	ret

proc_menu_0_nextbgm:
	mov hl, bgm_select
	cmp [hl], 6
	jz do_ret
	inc [hl]
	mov a, 8
	call wait_prc_num
	call render_menu_updatebgmname
	ret

proc_menu_1:
	; Set cursor
	xor a, a
	mov b, $1A
	mov [MAP_BASE + (1 * 12) + 1], a
	mov [MAP_BASE + (2 * 12) + 1], b
	mov [MAP_BASE + (3 * 12) + 1], a
	mov [MAP_BASE + (4 * 12) + 1], a
	; Process keys
	call read_keys
	tst a, KEY_LEFT
	jnz proc_menu_1_prevsfx
	tst a, KEY_RIGHT
	jnz proc_menu_1_nextsfx
	ret

proc_menu_1_prevsfx:
	mov hl, sfx_select
	cmp [hl], 0
	jz do_ret
	dec [hl]
	mov a, 8
	call wait_prc_num
	ret

proc_menu_1_nextsfx:
	mov hl, sfx_select
	cmp [hl], 4
	jz do_ret
	inc [hl]
	mov a, 8
	call wait_prc_num
	ret

proc_menu_2:
	; Set cursor
	xor a, a
	mov b, $1A
	mov [MAP_BASE + (1 * 12) + 1], a
	mov [MAP_BASE + (2 * 12) + 1], a
	mov [MAP_BASE + (3 * 12) + 1], b
	mov [MAP_BASE + (4 * 12) + 1], a
	; Get volume and write to map
	call pmmusic_getvolbgm
	mov b, 0
	mov hl, hexnum
	add hl, ba
	mov a, [hl]	
	mov [MAP_BASE + (3 * 12) + 10], a
	; Process keys
	call read_keys
	tst a, KEY_LEFT
	jnz proc_menu_2_decvol
	tst a, KEY_RIGHT
	jnz proc_menu_2_incvol
	ret

proc_menu_2_decvol:
	call pmmusic_getvolbgm
	dec a
	call pmmusic_setvolbgm
	mov a, 8
	call wait_prc_num
	ret

proc_menu_2_incvol:
	call pmmusic_getvolbgm
	inc a
	call pmmusic_setvolbgm
	mov a, 8
	call wait_prc_num
	ret

proc_menu_3:
	; Set cursor
	xor a, a
	mov b, $1A
	mov [MAP_BASE + (1 * 12) + 1], a
	mov [MAP_BASE + (2 * 12) + 1], a
	mov [MAP_BASE + (3 * 12) + 1], a
	mov [MAP_BASE + (4 * 12) + 1], b
	; Get volume and write to map
	call pmmusic_getvolsfx
	mov b, 0
	mov hl, hexnum
	add hl, ba
	mov a, [hl]	
	mov [MAP_BASE + (4 * 12) + 10], a
	; Process keys
	call read_keys
	tst a, KEY_LEFT
	jnz proc_menu_3_decvol
	tst a, KEY_RIGHT
	jnz proc_menu_3_incvol
	ret

proc_menu_3_decvol:
	call pmmusic_getvolsfx
	dec a
	call pmmusic_setvolsfx
	mov a, 8
	call wait_prc_num
	ret

proc_menu_3_incvol:
	call pmmusic_getvolsfx
	inc a
	call pmmusic_setvolsfx
	mov a, 8
	call wait_prc_num
	ret

; --------------------------------------------------------------------------------

hexnum:
	.db "0", "1", "2", "3", "4", "5", "6", "7"
	.db "8", "9", "A", "B", "C", "D", "E", "F"

.align 16
bgm_list:
	.dw music_test, music_test >> 16
	.db "SndEng Tst  "
	.dw music_pmintro, music_pmintro >> 16
	.db "PM Intro    "
	.dw music_choco, music_choco >> 16
	.db "OdekaChocobo"
	.dw bgm_unk_tune1, bgm_unk_tune1 >> 16
	.db "Unk. Tune 1 "
	.dw bgm_unk_tune2, bgm_unk_tune2 >> 16
	.db "Unk. Tune 2 "
	.dw music_galactix, music_galactix >> 16
	.db "Galactix1stP"
	.dw music_metroid, music_metroid >> 16
	.db "MetroidTitle"
sfx_list:
	.dw sound_fx1, sound_fx1 >> 16
	.dw sound_fx2, sound_fx2 >> 16
	.dw sound_fx3, sound_fx3 >> 16
	.dw sound_fx4, sound_fx4 >> 16
	.dw sound_fx5, sound_fx5 >> 16

.align 8
font8x8:
	.incbin "font8x8.bin"

.align 2
music_data_start:

.include "music_test.asm"
.include "music_pmintro.asm"
.include "music_choco.asm"
.include "music_unknown.asm"
.include "music_galactix.asm"
.include "music_metroid.asm"

sound_fx1:
	ROW N_C_4, $80, 3, 1
	ROW N_C_3, $80, 3, 1
	ROW N_C_5, $80, 3, 1
	ROW N_C_7, $80, 3, 1
	END_SOUND

sound_fx2:
	ROW N_C_4, $80, 3, 1
	ROW N_C_5, $70, 3, 1
	ROW N_C_4, $60, 3, 1
	ROW N_C_5, $50, 3, 1
	ROW N_C_4, $40, 3, 1
	ROW N_C_5, $30, 3, 1
	ROW N_C_4, $20, 3, 1
	ROW N_C_5, $10, 3, 1
	ROW N_C_4, $00, 3, 1
	END_SOUND

sound_fx3:
	ROW N_C_1, $80, 3, 1
	ROW N_E_1, $80, 3, 1
	ROW N_G_1, $80, 3, 1
	ROW N_C_2, $80, 3, 1
	ROW N_E_2, $80, 3, 1
	ROW N_G_2, $80, 3, 1
	ROW N_C_3, $80, 3, 1
	ROW N_E_3, $80, 3, 1
	ROW N_G_3, $80, 3, 1
	ROW N_C_4, $80, 3, 1
	ROW N_E_4, $80, 3, 1
	ROW N_G_4, $80, 3, 1
	END_SOUND

sound_fx4:
	ROW N_C_4, $40, 0, 1
	ROW N_C_4, $40, 3, 1
	ROW N_C_3, $40, 2, 1
	ROW N_C_2, $40, 2, 1
	ROW N_C_1, $40, 2, 1
	ROW N_C_1, $40, 2, 1
	END_SOUND

sound_fx5:
	ROW N_A_5, $40, 2, 6
	WRITE_RAM (0 * 12 + 10), $2B
	ROW N_A_5, $20, 2, 6
	WRITE_RAM (0 * 12 + 10), $2D
	END_SOUND

music_data_end:
	.printf "Music Data size: %i bytes\n", music_data_end - music_data_start

; --------------------------------------------------------------------------------

	ROMEND
