; Copyright (C) 2012 by JustBurn
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in
; all copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
; THE SOFTWARE.

.ifndef PM_CSTYLE
.error "pm_cstyle.s must be included"
.endif

.if . < 0x2200
.error "pmrom_cmath.s must be included after pm_header"
.elif . > 0xFFFF
.warning "pmrom_cmath.s requires to be in bank 0"
.endif


; ] Absolute value
; | c_abs num
; *   num - 16-bits number
; # return_to: absolute value
	declare_function "c_abs", 0, 1
.ifdef c_abs_cinvoked
	begin_function
	getparam 0, REG_BA
	tst b, $80
	returnc z
	mov hl, 0
	sub hl, ba
	mov ba, hl
	end_function 0
.endif
.macro c_abs num
	cinvoke1 c_abs, num
.endm


; ] Compute square root
; | c_sqrt num
; *   num - 16-bits integer
; # return_to: Square root of num
	declare_function "c_sqrt", 0, 1
.ifdef c_sqrt_cinvoked
	begin_function
	getparam 0, REG_BA
	mov x, ba
	mov b, 128
	xor a, a
:	add a, b
	mov l, a
	mul l, a
	cmp hl, x
	jcb :f
	jzb :f
	sub a, b
:	shr b
	cmp b, 0
	jnzb :bb
	end_function 0
.endif
.macro c_sqrt num
	cinvoke1 c_sqrt, num
.endm


; ] Initialize random number generator
; | c_srand num
; *   num - 16-bits integer
	declare_function "c_srand", 0, 1
.ifdef c_srand_cinvoked
	begin_function
	getparam_iba 0, RAM_OF+$14E0
	end_function 0
.endif
.macro c_srand num
	cinvoke1 c_srand, num
.endm


; ] Generate pseudo-random number
; | c_rand
; # return_to: Random number
	declare_function "c_rand", 0, 0
.ifdef c_rand_cinvoked
	begin_function
	mov i, 0
	mov ba, [$14E0]
	cinvoke2 c_umul, REG_BA, 31421
	add ba, 6927
	mov [$14E0], ba
	end_function 0
.endif
.macro c_rand
	cinvoke0 c_rand
.endm


; ] Generate random number fast
; | c_randf
; # return_to: Random number
	declare_function "c_randf", 0, 0
.ifdef c_randf_cinvoked
	begin_function
	mov i, 0
	mov ba, [$14E0]
	xor a, [n+SEC_CNT_LO]
	rol a
	xor a, [n+PRC_CNT]
	rol a
	xor a, [n+TMR256_CNT]
	rol a
	add ba, 6927
	sbc ba, 12345
	mov [$14E0], ba
	end_function 0
.endif
.macro c_randf
	cinvoke0 c_randf
.endm


; ] Return sine of an angle
; | c_sin angle
; *   angle - angle (0x0000 = 0, 0x0080 = 180, 0x0100 = 360)
; # return_to: sine value in fixed point 8.8
	declare_function "c_sin", 0, 1
.ifdef c_sin_cinvoked
	begin_function
	getparam 0, REG_L
	mov h, 0
	add hl, hl
	add hl, c_sin_table
	mov i, 0
	mov ba, [hl]
	end_function 0
.endif
.macro c_sin angle
	cinvoke1 c_sin, angle
.endm


; ] Return cosine of an angle
; | c_cos angle
; *   angle - angle (0x0000 = 0, 0x0080 = 180, 0x0100 = 360)
; # return_to: cosine value in fixed point 8.8
	declare_function "c_cos", 0, 1
.ifdef c_cos_cinvoked
	begin_function
	getparam 0, REG_A
	add a, $40
	mov l, a
	mov h, 0
	add hl, hl
	add hl, c_sin_table
	mov i, 0
	mov ba, [hl]
	end_function 0
.endif
.macro c_cos angle
	cinvoke1 c_cos, angle
.endm


; ] Return tangent of an angle
; | c_tan angle
; *   angle - angle (0x0000 = 0, 0x0080 = 180, 0x0100 = 360)
; # return_to: tangent value in fixed point 8.8
	declare_function "c_tan", 0, 1
.ifdef c_tan_cinvoked
	begin_function
	getparam 0, REG_L
	mov h, 0
	and l, $7F
	add hl, hl
	add hl, c_tan_table
	mov i, 0
	mov ba, [hl]
	end_function 0
.endif
.macro c_tan angle
	cinvoke1 c_tan, angle
.endm


; ] Compute arc sine
; | c_asin sinvalue
; * value - Sine value in fixed point 8.8
; # return_to: Angle (0x0000 = 0, 0x0040 = 90, 0xFFC0 = -90)
	declare_function "c_asin", 0, 1
.ifdef c_asin_cinvoked
	begin_function
	getparam 0, REG_BA
	cmp ba, $0100
	jg :ff
	cmp ba, $FF00
	jl :ff
	push b
	tst b, 0x80
	jzb :f
	mov hl, 0
	sub hl, ba
	mov ba, hl
:	cinvoke3 c_nisearch, REG_BA, c_sin_table, 128
	pop l
	and l, 0x80
	returnc z
	mov hl, 0
	sub hl, ba
	mov ba, hl
	return
:	mov ba, 0
	end_function 0
.endif
.macro c_asin sinvalue
	cinvoke1 c_asin, sinvalue
.endm


; ] Compute arc cosine
; | c_acos cosvalue
; * value - Cosine value in fixed point 8.8
; # return_to: Angle (0x0000 = 0, 0x0040 = 90, 0xFFC0 = -90)
	declare_function "c_acos", 0, 1
.ifdef c_acos_cinvoked
	begin_function
	getparam 0, REG_BA
	cmp ba, $0100
	jg :ff
	cmp ba, $FF00
	jl :ff
	push b
	tst b, 0x80
	jzb :f
	mov hl, 0
	sub hl, ba
	mov ba, hl
:	cinvoke3 c_ndsearch, REG_BA, c_sin_table, 256
	sub ba, $0040
	pop l
	and l, 0x80
	returnc z
	mov hl, 0
	sub hl, ba
	mov ba, hl
	return
:	mov ba, 0
	end_function 0
.endif
.macro c_acos cosvalue
	cinvoke1 c_acos, cosvalue
.endm


; ] Compute arc tangent
; | c_atan tanvalue
; * value - Tangent value in fixed point 8.8
; # return_to: Angle (0x0000 = 0, 0x0040 = 90, 0xFFC0 = -90)
	declare_function "c_atan", 0, 1
.ifdef c_atan_cinvoked
	begin_function
	getparam 0, REG_BA
	push b
	tst b, 0x80
	jzb :f
	mov hl, 0
	sub hl, ba
	mov ba, hl
:	cinvoke3 c_nisearch, REG_BA, c_tan_table, 128
	pop l
	and l, 0x80
	returnc z
	mov hl, 0
	sub hl, ba
	mov ba, hl
	end_function 0
.endif
.macro c_atan tanvalue
	cinvoke1 c_atan, tanvalue
.endm


; ] Compute arc tangent with two parameters
; | c_atan2 yval, xval
; * yval - Number representing the y-coordinate
; * xval - Number representing the x-coordinate
; # return_to: Angle (0x0000 = 0, 0x0080 = 180, 0xFF80 = -180)
	declare_function "c_atan2", 0, 2
.ifdef c_atan2_cinvoked
	begin_function
	mov i, 0
	getparam 0, REG_BA
	tst b, $80
	jzb :f
	mov x, 0
	sub x, ba
	mov ba, x
:	getparam 1, REG_HL
	and l, $FF
	jns :f
	mov x, 0
	sub x, hl
	mov hl, x
:	cmp ba, hl
	jncb _10
	cmp hl, 0
	jnzb _01
	xor a, a
	jmpb _20
_01:
	cmp b, 0
	jz :f
	mov a, 255
:	mov b, a
	xor a, a
	mov x, hl
	cinvoke2 c_udiv, REG_BA, REG_X
	mov hl, ba
	add hl, c_atan2_table
	mov a, [hl]
	jmpb _20
_10:
	cmp ba, 0
	jnzb _11
	mov a, 64
	jmpb _20
_11:
	cmp h, 0
	jz :f
	mov l, 255
:	mov h, l
	mov l, 0
	mov x, hl
	cinvoke2 c_udiv, REG_X, REG_BA
	mov hl, ba
	add hl, c_atan2_table
	mov b, [hl]
	mov a, 64
	sub a, b
_20:
	mov b, 0
	getparam 1, REG_HL
	cmp hl, 0
	js _30
	getparam 0, REG_HL
	cmp hl, 0
	returnc ns
	mov hl, 0
	sub hl, ba
	return_with REG_HL
_30:
	getparam 0, REG_HL
	cmp hl, 0
	js _31
	mov hl, 128
	sub hl, ba
	return_with REG_HL
_31:
	sub ba, 128
	end_function 0
.endif
.macro c_atan2 yval, xval
	cinvoke2 c_atan2, yval, xval
.endm


; ] Compute arc tangent with two parameters (8-bits values)
; | c_atan2b yval, xval
; * yval - 8-Bits number representing the y-coordinate
; * xval - 8-Bits number representing the x-coordinate
; # return_to: Angle (0x0000 = 0, 0x0080 = 180, 0xFF80 = -180)
	declare_function "c_atan2b", 0, 2
.ifdef c_atan2b_cinvoked
	begin_function
	mov i, 0
	getparam 0, REG_A
	cmp a, 0
	jns :f
	neg a
:	getparam 1, REG_B
	cmp b, 0
	jns :f
	neg b
:	cmp a, b
	jncb _10
	cmp b, 0
	jnzb _01
	xor a, a
	jmpb _20
_01:
	xchg a, b
	mov h, b
	mov l, 0
	div hl, a
	jno :f
	mov l, 255
:	mov h, 0
	add hl, c_atan2_table
	mov a, [hl]
	jmpb _20
_10:
	cmp a, 0
	jnzb _11
	mov a, 64
	jmpb _20
_11:
	mov h, b
	mov l, 0
	div hl, a
	jno :f
	mov l, 255
:	mov h, 0
	add hl, c_atan2_table
	mov b, [hl]
	mov a, 64
	sub a, b
_20:
	getparam 1, REG_B
	cmp b, 0
	js _30
	getparam 0, REG_B
	cmp b, 0
	returnc_with ns, REG_A
	xchg a, b
	mov a, 0
	sub a, b
	mov b, 255
	return
_30:
	getparam 0, REG_B
	cmp b, 0
	js _31
	xchg a, b
	mov a, 128
	sub a, b
	return_with REG_A
_31:
	sub a, 128
	mov b, 255
	end_function 0
.endif
.macro c_atan2b yval, xval
	cinvoke2 c_atan2b, yval, xval
.endm


; ] Unsigned 16-bits multiplication
; | c_umul multiplicand, multiplier
; # return_to: Product of multiplication
	declare_function "c_umul", 0, 2
.ifdef c_umul_cinvoked
	begin_function
	getparam 1, REG_BA
	getparam 0, REG_HL
	mul l, a
	push hl
	getparam 0, REG_UL
	mul l, a
	mov a, b
	mov b, l
	getparam 0, REG_L
	mul l, a
	mov a, l
	add a, b
	mov b, a
	xor a, a
	pop hl
	add ba, hl
	end_function 0
.endif
.macro c_umul multiplicand, multiplier
	cinvoke2 c_umul, multiplicand, multiplier
.endm


; ] Signed 16-bits multiplication
; | c_smul multiplicand, multiplier
; # return_to: Product of multiplication
	declare_function "c_smul", 0, 2
.ifdef c_smul_cinvoked
	begin_function
	xor a, a
	mov x, 0
	getparam 1, REG_UB
	tst b, $80
	jzb :f
	xor a, $01
	getparam 1, REG_HL
	sub x, hl
	mov hl, x
	setparam 1, REG_HL
:	getparam 0, REG_HL
	mov b, h
	tst b, $80
	jzb :f
	xor a, $01
	sub x, hl
	mov hl, x
	setparam 0, REG_HL
:	push a
	getparam 1, REG_BA
	mul l, a
	push hl
	getparam 0, REG_UL
	mul l, a
	mov a, b
	mov b, l
	getparam 0, REG_L
	mul l, a
	mov a, l
	add a, b
	mov b, a
	xor a, a
	pop hl
	add ba, hl
	pop l
	and l, $01
	returnc z
	mov x, 0
	sub x, ba
	mov ba, x
	end_function 0
.endif
.macro c_smul multiplicand, multiplier
	cinvoke2 c_smul, multiplicand, multiplier
.endm


; ] Unsigned 16-bits division
; | c_udiv dividend, divisor
; # return_to: Quotient of division
	declare_function "c_udiv", 0, 2
.ifdef c_udiv_cinvoked
	begin_function
	mov x, 16
	getparam_iba 0, REG_HL
	getparam_iba 1, REG_Y_DESTROY
	mov ba, 0
	cmp y, 0
	returnc z
:	add hl, hl
	rolc a
	rolc b
	sub ba, y
	jncb :f
	add ba, y
	dec x
	jnzb :b
	jmpb :ff
:	inc hl
	dec x
	jnzb :bb
:	mov ba, hl
	end_function 0
.endif
.macro c_udiv dividend, divisor
	cinvoke2 c_udiv, dividend, divisor
.endm


; ] Signed 16-bits division
; | c_smul dividend, divisor
; # return_to: Quotient of division
	declare_function "c_sdiv", 0, 2
.ifdef c_sdiv_cinvoked
	begin_function
	xor a, a
	mov x, 0
	getparam 1, REG_UB
	tst b, $80
	jzb :f
	xor a, $01
	getparam 1, REG_HL
	sub x, hl
	mov hl, x
	setparam 1, REG_HL
:	getparam 0, REG_HL
	mov b, h
	tst b, $80
	jzb :f
	xor a, $01
	sub x, hl
	mov hl, x
:	push a
	mov x, 16
	getparam_iba 1, REG_Y_DESTROY
	mov ba, 0
	cmp y, 0
	returnc z
:	add hl, hl
	rolc a
	rolc b
	sub ba, y
	jncb :f
	add ba, y
	dec x
	jnzb :b
	jmpb :ff
:	inc hl
	dec x
	jnzb :bb
:	mov ba, hl
	pop l
	and l, $01
	returnc z
	mov x, 0
	sub x, ba
	mov ba, x
	end_function 0
.endif
.macro c_sdiv dividend, divisor
	cinvoke2 c_sdiv, dividend, divisor
.endm


; ] Unsigned 16-bits modulus
; | c_umod dividend, divisor
; # return_to: Remainer of division
;           X: Quotient of division
	declare_function "c_umod", 0, 2
.ifdef c_umod_cinvoked
	begin_function
	mov x, 16
	getparam_iba 0, REG_HL
	getparam_iba 1, REG_Y_DESTROY
	mov ba, 0
:	add hl, hl
	rolc a
	rolc b
	sub ba, y
	jncb :f
	add ba, y
	dec x
	jnzb :b
	mov x, hl
	return
:	inc hl
	dec x
	jnzb :bb
	mov x, hl
	end_function 0
.endif
.macro c_umod dividend, divisor
	cinvoke2 c_umod, dividend, divisor
.endm


; ] Signed 16-bits modulus
; | c_smod dividend, divisor
; # return_to: Remainer of multiplication
	declare_function "c_smod", 0, 2
.ifdef c_smod_cinvoked
	begin_function
	xor a, a
	mov x, 0
	getparam 1, REG_UB
	tst b, $80
	jzb :f
	inc a
	getparam 1, REG_HL
	sub x, hl
	mov hl, x
	setparam 1, REG_HL
:	getparam 0, REG_HL
	mov b, h
	tst b, $80
	jzb :f
	sub x, hl
	mov hl, x
:	push a
	mov x, 16
	getparam_iba 1, REG_Y_DESTROY
	mov ba, 0
:	add hl, hl
	rolc a
	rolc b
	sub ba, y
	jncb :f
	add ba, y
	dec x
	jnzb :b
	jmpb :ff
:	inc hl
	dec x
	jnzb :bb
:	pop l
	and l, $01
	returnc z
	mov y, 0
	sub y, ba
	mov ba, y
	end_function 0
.endif
.macro c_smod dividend, divisor
	cinvoke2 c_smod, dividend, divisor
.endm


; ] Number search in incrementing array
; | c_nisearch nkey, table_addr, sizen
; *   nkey       - Number key for the search
; *   table_addr - Array table of 16-bits to search
; *   sizen      - Size of the table in bytes
	declare_function "c_nisearch", PTR_2, 4
.ifdef c_nisearch_cinvoked
	begin_function
	getparam 2, REG_A
	mov xi, a
	getparam_ihl 1, REG_X
	getparam_ihl 3, REG_BA
	getparam_ihl 0, REG_Y_DESTROY
	push x
	shr b
	rorc a
	and a, $FE
	add x, ba
_loop:	shr b
	rorc a
	and a, $FE
	mov hl, [x]
	cmp hl, y
	jzb _end
	jcb :f
	sub x, ba
	jmpb :ff
:	add x, ba
:	cmp ba, 0
	jnz _loop
_end:
	pop ba
	sub x, ba
	mov ba, x
	shr b
	rorc a
	end_function 0
.endif
.macro c_nisearch nkey, table_addr, sizen
	cinvoke3 c_nisearch, nkey, table_addr, sizen
.endm


; ] Number search in decrementing array
; | c_ndsearch nkey, table_addr, sizen
; *   nkey       - Number key for the search
; *   table_addr - Array table of 16-bits to search
; *   sizen      - Size of the table in bytes
	declare_function "c_ndsearch", PTR_2, 4
.ifdef c_ndsearch_cinvoked
	begin_function
	getparam 2, REG_A
	mov xi, a
	getparam_ihl 1, REG_X
	getparam_ihl 3, REG_BA
	getparam_ihl 0, REG_Y_DESTROY
	push x
	shr b
	rorc a
	and a, $FE
	add x, ba
_loop:	shr b
	rorc a
	and a, $FE
	mov hl, [x]
	cmp hl, y
	jzb _end
	jcb :f
	add x, ba
	jmpb :ff
:	sub x, ba
:	cmp ba, 0
	jnz _loop
_end:
	pop ba
	sub x, ba
	mov ba, x
	shr b
	rorc a
	end_function 0
.endif
.macro c_ndsearch nkey, table_addr, sizen
	cinvoke3 c_ndsearch, nkey, table_addr, sizen
.endm


.set c_sin_table_available 0
.ifdef c_sin_cinvoked
.set c_sin_table_available 1
.endif
.ifdef c_cos_cinvoked
.set c_sin_table_available 1
.endif
.ifdef c_asin_cinvoked
.set c_sin_table_available 1
.endif
.ifdef c_acos_cinvoked
.set c_sin_table_available 1
.endif

.if (c_sin_table_available == 1)
c_sin_table:
	.dw $0000,$0006,$000D,$0013,$0019,$001F,$0026,$002C,$0032,$0038,$003E,$0044,$004A,$0050,$0056,$005C
	.dw $0062,$0068,$006D,$0073,$0079,$007E,$0084,$0089,$008E,$0093,$0098,$009D,$00A2,$00A7,$00AC,$00B1
	.dw $00B5,$00B9,$00BE,$00C2,$00C6,$00CA,$00CE,$00D1,$00D5,$00D8,$00DC,$00DF,$00E2,$00E5,$00E7,$00EA
	.dw $00ED,$00EF,$00F1,$00F3,$00F5,$00F7,$00F8,$00FA,$00FB,$00FC,$00FD,$00FE,$00FE,$00FF,$00FF,$00FF
	.dw $0100,$0100,$0100,$00FF,$00FF,$00FE,$00FD,$00FC,$00FB,$00FA,$00F8,$00F7,$00F5,$00F3,$00F1,$00EF
	.dw $00ED,$00EA,$00E7,$00E5,$00E2,$00DF,$00DC,$00D8,$00D5,$00D1,$00CE,$00CA,$00C6,$00C2,$00BE,$00B9
	.dw $00B5,$00B1,$00AC,$00A7,$00A2,$009D,$0098,$0093,$008E,$0089,$0084,$007E,$0079,$0073,$006D,$0068
	.dw $0062,$005C,$0056,$0050,$004A,$0044,$003E,$0038,$0032,$002C,$0026,$001F,$0019,$0013,$000D,$0006
	.dw $0000,$FFFA,$FFF3,$FFED,$FFE7,$FFE1,$FFDA,$FFD4,$FFCE,$FFC8,$FFC2,$FFBC,$FFB6,$FFB0,$FFAA,$FFA4
	.dw $FF9E,$FF98,$FF93,$FF8D,$FF87,$FF82,$FF7C,$FF77,$FF72,$FF6D,$FF68,$FF63,$FF5E,$FF59,$FF54,$FF4F
	.dw $FF4B,$FF47,$FF42,$FF3E,$FF3A,$FF36,$FF32,$FF2F,$FF2B,$FF28,$FF24,$FF21,$FF1E,$FF1B,$FF19,$FF16
	.dw $FF13,$FF11,$FF0F,$FF0D,$FF0B,$FF09,$FF08,$FF06,$FF05,$FF04,$FF03,$FF02,$FF02,$FF01,$FF01,$FF01
	.dw $FF00,$FF00,$FF00,$FF01,$FF01,$FF02,$FF03,$FF04,$FF05,$FF06,$FF08,$FF09,$FF0B,$FF0D,$FF0F,$FF11
	.dw $FF13,$FF16,$FF19,$FF1B,$FF1E,$FF21,$FF24,$FF28,$FF2B,$FF2F,$FF32,$FF36,$FF3A,$FF3E,$FF42,$FF47
	.dw $FF4B,$FF4F,$FF54,$FF59,$FF5E,$FF63,$FF68,$FF6D,$FF72,$FF77,$FF7C,$FF82,$FF87,$FF8D,$FF93,$FF98
	.dw $FF9E,$FFA4,$FFAA,$FFB0,$FFB6,$FFBC,$FFC2,$FFC8,$FFCE,$FFD4,$FFDA,$FFE1,$FFE7,$FFED,$FFF3,$FFFA
.endif


.set c_tan_table_available 0
.ifdef c_tan_cinvoked
.set c_tan_table_available 1
.endif
.ifdef c_atan_cinvoked
.set c_tan_table_available 1
.endif

.if (c_tan_table_available == 1)
c_tan_table:
	.dw $0000,$0006,$000D,$0013,$0019,$0020,$0026,$002C,$0033,$0039,$0040,$0047,$004E,$0055,$005C,$0063
	.dw $006A,$0071,$0079,$0081,$0089,$0091,$0099,$00A2,$00AB,$00B4,$00BE,$00C8,$00D2,$00DD,$00E8,$00F4
	.dw $0100,$010D,$011A,$0129,$0138,$0148,$0159,$016B,$017F,$0194,$01AB,$01C4,$01DF,$01FD,$021D,$0242
	.dw $026A,$0298,$02CB,$0307,$034C,$039D,$03FE,$0474,$0507,$05C3,$06BE,$081C,$0A27,$0D8F,$145B,$28BC
	.dw $7FFF,$D744,$EBA5,$F271,$F5D9,$F7E4,$F942,$FA3D,$FAF9,$FB8C,$FC02,$FC63,$FCB4,$FCF9,$FD35,$FD68
	.dw $FD96,$FDBE,$FDE3,$FE03,$FE21,$FE3C,$FE55,$FE6C,$FE81,$FE95,$FEA7,$FEB8,$FEC8,$FED7,$FEE6,$FEF3
	.dw $FF00,$FF0C,$FF18,$FF23,$FF2E,$FF38,$FF42,$FF4C,$FF55,$FF5E,$FF67,$FF6F,$FF77,$FF7F,$FF87,$FF8F
	.dw $FF96,$FF9D,$FFA4,$FFAB,$FFB2,$FFB9,$FFC0,$FFC7,$FFCD,$FFD4,$FFDA,$FFE0,$FFE7,$FFED,$FFF3,$FFFA
.endif


.set c_atan2_table_available 0
.ifdef c_atan2_cinvoked
.set c_atan2_table_available 1
.endif
.ifdef c_atan2b_cinvoked
.set c_atan2_table_available 1
.endif

.if (c_atan2_table_available == 1)
c_atan2_table:
	.db $00,$00,$00,$00,$01,$01,$01,$01,$01,$01,$02,$02,$02,$02,$02,$02
	.db $03,$03,$03,$03,$03,$03,$03,$04,$04,$04,$04,$04,$04,$05,$05,$05
	.db $05,$05,$05,$06,$06,$06,$06,$06,$06,$06,$07,$07,$07,$07,$07,$07
	.db $08,$08,$08,$08,$08,$08,$08,$09,$09,$09,$09,$09,$09,$0A,$0A,$0A
	.db $0A,$0A,$0A,$0A,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0C,$0C,$0C,$0C,$0C
	.db $0C,$0C,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0E,$0E,$0E,$0E,$0E,$0E,$0E
	.db $0F,$0F,$0F,$0F,$0F,$0F,$0F,$10,$10,$10,$10,$10,$10,$10,$11,$11
	.db $11,$11,$11,$11,$11,$11,$12,$12,$12,$12,$12,$12,$12,$13,$13,$13
	.db $13,$13,$13,$13,$13,$14,$14,$14,$14,$14,$14,$14,$14,$15,$15,$15
	.db $15,$15,$15,$15,$15,$15,$16,$16,$16,$16,$16,$16,$16,$16,$17,$17
	.db $17,$17,$17,$17,$17,$17,$17,$18,$18,$18,$18,$18,$18,$18,$18,$18
	.db $19,$19,$19,$19,$19,$19,$19,$19,$19,$19,$1A,$1A,$1A,$1A,$1A,$1A
	.db $1A,$1A,$1A,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1C,$1C,$1C
	.db $1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D
	.db $1D,$1D,$1D,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1F,$1F
	.db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$20,$20,$20,$20,$20,$20
	.db $20
.endif
